#compdef mkosi
# SPDX-License-Identifier: LGPL-2.1-or-later
# shellcheck shell=zsh

_mkosi_verb(){
    if (( CURRENT == 1 )); then
        _describe -t commands 'mkosi verb' _mkosi_verbs
    else
        local curcontext="$curcontext"
        cmd="${${_mkosi_verbs[(r)$words[1]:*]%%:*}}"
        if (( $#cmd )); then
            if (( $+functions[_mkosi_$cmd] )); then
                _mkosi_$cmd
            else
                _message "no more options"
            fi
        else
            _message "unknown mkosi verb: $words[1]"
        fi
    fi
}

declare -a _mkosi_verbs=(build clean summary shell boot qemu ssh serve bump help genkey documentation journalctl coredumpctl burn dependencies completion)

_arguments -s \
    '(- *)'{-h,--help}'[Show this help]' \
    '(- *)--version[Show package version]' \
    '(-f --force)'{-f,--force}'[Remove existing image file before operation]' \
    '(-C --directory)'{-C,--directory}'[Change to specified directory before doing anything]:directory:_files -f' \
    '--debug[Turn on debugging output]' \
    '--debug-shell[Spawn an interactive shell in the image if a chroot command fails]' \
    '--debug-workspace[When an error occurs, the workspace directory will not be deleted]' \
    '--no-pager[Enable paging for long output]' \
    '--genkey-valid-days[Number of days keys should be valid when generating keys]' \
    '--genkey-common-name[Template for the CN when generating keys]' \
    '(-B --auto-bump)'{-B,--auto-bump}'[Automatically bump image version after building]' \
    '--doc-format[The format to show documentation in]:arg:(auto markdown man pandoc system)' \
    '--json[Show summary as JSON]' \
    '(-I --include)'{-I,--include}'[Include configuration from the specified file or directory]' \
    '--initrd-include[Include configuration from the specified file or directory when building the initrd]' \
    '--profile[Build the specified profile]' \
    '--dependency[Specify other images that this image depends on]' \
    '--minimum-version[Specify the minimum required mkosi version]' \
    '--configure-script[Configure script to run before doing anything]' \
    '--pass-environment[Environment variables to pass to subimages]' \
    '(-d --distribution)'{-d,--distribution}'[Distribution to install]:arg:(fedora debian ubuntu arch opensuse mageia centos rhel rhel-ubi openmandriva rocky alma custom )' \
    '(-r --release)'{-r,--release}'[Distribution release to install]' \
    '--architecture[Override the architecture of installation]:arg:(alpha arc arm arm64 ia64 loongarch64 mips-le mips64-le parisc ppc ppc64 ppc64-le riscv32 riscv64 s390 s390x tilegx x86 x86-64 )' \
    '(-m --mirror)'{-m,--mirror}'[Distribution mirror to use]' \
    '--local-mirror[Use a single local, flat and plain mirror to build the image]' \
    '--repository-key-check[Controls signature and key checks on repositories]' \
    '--repositories[Repositories to use]' \
    '--cache-only[Only use the package cache when installing packages]:arg:(always auto none metadata never )' \
    '--package-manager-tree[Use a package manager tree to configure the package manager]' \
    '(-t --format)'{-t,--format}'[Output Format]:arg:(confext cpio directory disk esp none portable sysext tar uki oci )' \
    '--manifest-format[Manifest Format]' \
    '(-o --output)'{-o,--output}'[Output name]' \
    '--compress-output[Enable whole-output compression (with images or archives)]' \
    '--compress-level[Set the compression level to use]' \
    '(-O --output-dir)'{-O,--output-dir}'[Output directory]' \
    '--workspace-dir[Workspace directory]' \
    '--cache-dir[Incremental cache directory]' \
    '--package-cache-dir[Package cache directory]' \
    '--build-dir[Path to use as persistent build directory]' \
    '--image-version[Set version for image]' \
    '--image-id[Set ID for image]' \
    '--split-artifacts[Generate split partitions]' \
    '--repart-dir[Directory containing systemd-repart partition definitions]' \
    '--sector-size[Set the disk image sector size]' \
    '--repart-offline[Build disk images without using loopback devices]' \
    '--overlay[Only output the additions on top of the given base trees]' \
    '--use-subvolumes[Use btrfs subvolumes for faster directory operations where possible]' \
    '--seed[Set the seed for systemd-repart]' \
    '--clean-script[Clean script to run after cleanup]' \
    '(-p --package)'{-p,--package}'[Add an additional package to the OS image]' \
    '--build-package[Additional packages needed for build scripts]' \
    '--volatile-package[Packages to install after executing build scripts]' \
    '--package-directory[Specify a directory containing extra packages]' \
    '--volatile-package-directory[Specify a directory containing extra volatile packages]' \
    '--with-recommends[Install recommended packages]' \
    '--with-docs[Install documentation]' \
    '--base-tree[Use the given tree as base tree (e.g. lower sysext layer)]' \
    '--skeleton-tree[Use a skeleton tree to bootstrap the image before installing anything]' \
    '--extra-tree[Copy an extra tree on top of image]' \
    '--remove-package[Remove package from the image OS image after installation]' \
    '--remove-files[Remove files from built image]' \
    '--clean-package-metadata[Remove package manager database and other files]' \
    '--source-date-epoch[Set the $SOURCE_DATE_EPOCH timestamp]' \
    '--sync-script[Sync script to run before starting the build]' \
    '--prepare-script[Prepare script to run inside the image before it is cached]' \
    '--build-script[Build script to run inside image]' \
    '--postinst-script[Postinstall script to run inside image]' \
    '--finalize-script[Postinstall script to run outside image]' \
    '--postoutput-script[Output postprocessing script to run outside image]' \
    '--build-sources[Path for sources to build]' \
    '--build-sources-ephemeral[Make build sources ephemeral when running scripts]' \
    '(-E --environment)'{-E,--environment}'[Set an environment variable when running scripts]' \
    '--env-file[Enviroment files to set when running scripts]' \
    '(-T --without-tests)'{-T,--without-tests}'[Do not run tests as part of build scripts, if supported]' \
    '--with-network[Run build and postinst scripts with network access (instead of private network)]' \
    '--bootable[Generate ESP partition with systemd-boot and UKIs for installed kernels]' \
    '--bootloader[Specify which UEFI bootloader to use]:arg:(none uki systemd-boot grub )' \
    '--bios-bootloader[Specify which BIOS bootloader to use]:arg:(none grub )' \
    '--shim-bootloader[Specify whether to use shim]:arg:(none signed unsigned )' \
    '--unified-kernel-images[Specify whether to use UKIs with grub/systemd-boot in UEFI mode]' \
    '--unified-kernel-image-format[Specify the format used for the UKI filename]' \
    '--initrd[Add a user-provided initrd to image]' \
    $'--microcode-host[Only include the host CPU\'s microcode]' \
    '--initrd-package[Add additional packages to the default initrd]' \
    '--initrd-volatile-package[Packages to install in the initrd that are not cached]' \
    '--kernel-command-line[Set the kernel command line (only bootable images)]' \
    '--kernel-modules-include[Include the specified kernel modules in the image]' \
    '--kernel-modules-include-host[Include the currently loaded modules on the host in the image]' \
    '--kernel-modules-exclude[Exclude the specified kernel modules from the image]' \
    '--kernel-modules-initrd[When building a bootable image, add an extra initrd containing the kernel modules]' \
    '--kernel-modules-initrd-include[When building a kernel modules initrd, include the specified kernel modules]' \
    '--kernel-modules-initrd-include-host[When building a kernel modules initrd, include the currently loaded modules on the host in the image]' \
    '--kernel-modules-initrd-exclude[When building a kernel modules initrd, exclude the specified kernel modules]' \
    '--locale[Set the system locale]' \
    '--locale-messages[Set the messages locale]' \
    '--keymap[Set the system keymap]' \
    '--timezone[Set the system timezone]' \
    '--hostname[Set the system hostname]' \
    '--root-password[Set the password for root]' \
    '--root-shell[Set the shell for root]' \
    '(-a --autologin)'{-a,--autologin}'[Enable root autologin]' \
    '--make-initrd[Make sure the image can be used as an initramfs]' \
    $'--ssh[Set up SSH access from the host to the final image via \'mkosi ssh\']' \
    '--selinux-relabel[Specify whether to relabel all files with setfiles]' \
    '--secure-boot[Sign the resulting kernel/initrd image for UEFI SecureBoot]' \
    '--secure-boot-auto-enroll[Automatically enroll the secureboot signing key on first boot]' \
    '--secure-boot-key[UEFI SecureBoot private key]' \
    '--secure-boot-key-source[The source to use to retrieve the secure boot signing key]' \
    '--secure-boot-certificate[UEFI SecureBoot certificate in X509 format]' \
    '--secure-boot-sign-tool[Tool to use for signing PE binaries for secure boot]:arg:(auto sbsign pesign )' \
    '--verity-key[Private key for signing verity signature]' \
    '--verity-key-source[The source to use to retrieve the verity signing key]' \
    '--verity-certificate[Certificate for signing verity signature in X509 format]' \
    '--sign-expected-pcr[Measure the components of the unified kernel image (UKI) and embed the PCR signature into the UKI]' \
    '--passphrase[Path to a file containing the passphrase to use when LUKS encryption is selected]' \
    '--checksum[Write SHA256SUMS file]' \
    '--sign[Write and sign SHA256SUMS file]' \
    '--key[GPG key to use for signing]' \
    '--proxy-url[Set the proxy to use]' \
    $'--proxy-exclude[Don\'t use the configured proxy for the specified host(s)]' \
    '--proxy-peer-certificate[Set the proxy peer certificate]' \
    '--proxy-client-certificate[Set the proxy client certificate]' \
    '--proxy-client-key[Set the proxy client key]' \
    '(-i --incremental)'{-i,--incremental}'[Make use of and generate intermediary cache images]' \
    '--settings[Add in .nspawn settings file]' \
    '--extra-search-path[List of comma-separated paths to look for programs before looking in PATH]' \
    '--ephemeral[If specified, the container/VM is run with a temporary snapshot of the output image that is removed immediately when the container/VM terminates]' \
    '--credential[Pass a systemd credential to systemd-nspawn or qemu]' \
    '--kernel-command-line-extra[Append extra entries to the kernel command line when booting the image]' \
    '--acl[Set ACLs on generated directories to permit the user running mkosi to remove them]' \
    '--tools-tree[Look up programs to execute inside the given tree]' \
    '--tools-tree-distribution[Set the distribution to use for the default tools tree]:arg:(fedora debian ubuntu arch opensuse mageia centos rhel rhel-ubi openmandriva rocky alma custom )' \
    '--tools-tree-release[Set the release to use for the default tools tree]' \
    '--tools-tree-mirror[Set the mirror to use for the default tools tree]' \
    '--tools-tree-repository[Repositories to use for the default tools tree]' \
    '--tools-tree-package-manager-tree[Package manager trees for the default tools tree]' \
    '--tools-tree-package[Add additional packages to the default tools tree]' \
    '--tools-tree-certificates[Use certificates from the tools tree]' \
    '--runtime-tree[Additional mounts to add when booting the image]' \
    '--runtime-size[Grow disk images to the specified size before booting them]' \
    '--runtime-scratch[Mount extra scratch space to /var/tmp]' \
    '--runtime-network[Set networking backend to use when booting the image]:arg:(interface user none )' \
    '--runtime-build-sources[Mount build sources and build directory in /work when booting the image]' \
    '--unit-property[Set properties on the scopes spawned by systemd-nspawn or systemd-run]' \
    '--ssh-key[Private key for use with mkosi ssh in PEM format]' \
    '--ssh-certificate[Certificate for use with mkosi ssh in X509 format]' \
    '--vmm[Set the virtual machine monitor to use for mkosi qemu]:arg:(qemu vmspawn )' \
    '--machine[Set the machine name to use when booting the image]' \
    '--forward-journal[Set the path used to store forwarded machine journals]' \
    '--qemu-gui[Start QEMU in graphical mode]' \
    $'--qemu-smp[Configure guest\'s SMP settings]' \
    $'--qemu-mem[Configure guest\'s RAM size]' \
    '--qemu-kvm[Configure whether to use KVM or not]' \
    '--qemu-vsock[Configure whether to use qemu with a vsock or not]' \
    '--qemu-vsock-cid[Specify the VSock connection ID to use]' \
    '--qemu-swtpm[Configure whether to use qemu with swtpm or not]' \
    '--qemu-cdrom[Attach the image as a CD-ROM to the virtual machine]' \
    '--qemu-firmware[Set qemu firmware to use]:arg:(auto linux uefi uefi-secure-boot bios )' \
    '--qemu-firmware-variables[Set the path to the qemu firmware variables file to use]' \
    '--qemu-kernel[Specify the kernel to use for qemu direct kernel boot]' \
    '--qemu-drive[Specify a qemu drive that mkosi should create and pass to qemu]' \
    '--qemu-args[==SUPPRESS==]' \
    '*::mkosi verb:_mkosi_verb'


